@@ -1,6 +1,6 @@
 package Plack::App::Directory::Apaxy;
 {
-  $Plack::App::Directory::Apaxy::VERSION = '0.002';
+  $Plack::App::Directory::Apaxy::VERSION = '0.003';
 }
 # ABSTRACT: Serve static files from document root with directory index using Apaxy
 
@@ -188,8 +188,16 @@ sub call {
     my $self = shift;
     my $env  = shift;
 
-    my ( $file, $path_info ) = $self->file || $self->locate_apaxy($env) || $self->locate_file($env);
-    return $file if ref $file eq 'ARRAY';
+    my ( $file, $path_info ) = $self->file;
+    if ( !$file ) {
+        ( $file, $path_info ) = $self->locate_file($env);
+        if ( ref $file eq 'ARRAY' ) {
+            return $file unless $file->[0] == 404;
+
+            ( $file, $path_info ) = $self->locate_apaxy($env);
+            return $file if ref $file eq 'ARRAY';
+        }
+    }
 
     if ($path_info) {
         $env->{'plack.file.SCRIPT_NAME'} = $env->{SCRIPT_NAME} . $env->{PATH_INFO};
@@ -209,8 +217,8 @@ sub locate_apaxy {
     my ( $self, $env ) = @_;
 
     my $path = $env->{PATH_INFO} || q{};
-    return if     $path =~ m{\0};
-    return unless $path =~ m{^/_apaxy/};
+    return $self->return_400 if     $path =~ m{\0};
+    return $self->return_404 unless $path =~ m{^(/_apaxy/|/favicon.ico$)};
 
     my $docroot = $self->apaxy_root;
     my @path = split /[\\\/]/, $path;
@@ -220,31 +228,27 @@ sub locate_apaxy {
     else {
         @path = (q{.});
     }
-    return if grep $_ eq q{..}, @path;
+    return $self->return_403 if grep $_ eq q{..}, @path;
 
     my $file = path( $docroot, @path );
-    return unless $self->should_handle($file);
-    return unless -r $file;
+    return $self->return_404 unless $self->should_handle($file);
+    return $self->return_403 unless -r $file;
 
     return $file, join( q{/}, q{}, @path );
 }
 
-sub get_mime_type_icon {
-    my $self = shift;
-}
-
 # Stolen from Plack::App::Directory (Plack version 1.0030)
 sub serve_path {
     my ( $self, $env, $dir ) = @_;
 
-    if ( $dir =~ m{^/_apaxy/} ) {
+    return $self->SUPER::serve_path( $env, $dir ) if -e $dir;
+
+    if ( $dir =~ m{^(/_apaxy/|/favicon.ico$)} ) {
         my $docroot = $self->apaxy_root;
         my $file    = path( $docroot, $dir );
-        return $self->SUPER::serve_path( $env, $file ) if -f $file;
+        return $self->SUPER::serve_path( $env, $file ) if -e $file;
     }
 
-    return $self->SUPER::serve_path( $env, $dir ) if -f $dir;
-
     my $dir_url = $env->{SCRIPT_NAME} . $env->{PATH_INFO};
     return $self->return_dir_redirect($env) if $dir_url !~ m{/$};
 
@@ -404,7 +408,7 @@ Plack::App::Directory::Apaxy - Serve static files from document root with direct
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
